@@ -1,6 +1,6 @@
 /* 
 Gregorio messages headers.
-Copyright (C) 2006 Elie Roux
+Copyright (C) 2006-2009 Elie Roux <elie.roux@telecom-bretagne.eu>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,6 +20,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #ifndef MESSAGES_H
 #define MESSAGES_H
 
+#ifndef ENABLE_NLS
+#define ENABLE_NLS 0
+#endif
+#if ENABLE_NLS == 1
+    #include "gettext.h"
+    #define _(str) gettext(str)
+    #define N_(str) str
+    #define ngt_(str, strtwo, count) ngettext(str, strtwo, count)
+#else
+    #define _(str) str
+    #define N_(str) str
+    #define ngt_(str, strtwo, count) str
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
