@@ -16,29 +16,29 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/*
-
-This file contains almost all the gregorio_character manipulation things.
-
-The code is very spaghetti, but I don't really think I can do it another way with the chosen representation of styles, and the fact that everything must be xml-compliant and tex-compliant...
-
-So this file is basically very hard to maintain. Moreover, it has not been very well coded, so it is even harder to maintain...
-
-*/
+/**
+ * @file
+ * @brief This file contains almost all the gregorio_character manipulation things.
+ *
+ * @warning The code is very spaghetti, but I don't really think I can do it another way with the chosen representation of styles, and the fact that everything must be xml-compliant and tex-compliant...
+ * So this file is basically very hard to maintain. Moreover, it has not been very well coded, so it is even harder to maintain...
+ */
 
 #include "config.h"
 #include <stdio.h>
-#include "gettext.h"
 #include <stdlib.h>
 #include <gregorio/struct.h>
 #include <gregorio/unicode.h>
 #include <gregorio/characters.h>
 #include <gregorio/messages.h>
-#define _(str) gettext(str)
-#define N_(str) str
-
-/* Here is a function that tests if a letter is a vowel or not */
 
+/*!
+ * @brief Tests if a letter is a vowel or not. 
+ * 
+ * Necessary for determining proper centering, as we want the first neume centered over the vowel. Simple, we just compare \c letter to a list of vowels used in Latin, both accented and not.
+ * \param letter The letter being tested
+ * \return returns \c 1 if a vowel; otherwise returns \c 0
+ */
 int
 gregorio_is_vowel (grewchar letter)
 {
@@ -59,7 +59,6 @@ gregorio_is_vowel (grewchar letter)
 }
 
 // a macro that will be used for verbatim and special-characters in the next function, it calls function with a grewchar * which is the verbatim or special-character. It places current_character to the character next to the end of the verbatim or special_char charachters.
-
 #define verb_or_sp(ST_TYPE, function) \
 		  i = 0;\
 		  j = 0;\
@@ -109,18 +108,13 @@ gregorio_is_vowel (grewchar letter)
 
 
 
-/*
-
-This function is made to simplify the output modules : they just have to declare some simple functions (what to do when meeting a beginning of style, a character, etc.) and to call this function with pointer to these functions, and that will automatically write the good ouput. This function does not test at all the gregorio_character list, if it is wrong, then the ouput will be wrong. It is very simple to understand, even if it is a bit long.
-
-type may be 0, or SKIP_FIRST_LETTER
-
-The difficulty comes when we have to write the first syllable text, without the first letter.
-
-The behaviour can have some bugs is this case if the first syllable has some complex styles. It would be a bit stupid to so such a thing, but users are usually very creative when it comes to invent twisted things...
-
-*/
-
+/**
+ * This function is made to simplify the output modules : they just have to declare some simple functions (what to do when meeting a beginning of style, a character, etc.) and to call this function with pointer to these functions, and that will automatically write the good ouput. This function does not test at all the gregorio_character list, if it is wrong, then the ouput will be wrong. It is very simple to understand, even if it is a bit long.
+ * type may be 0, or SKIP_FIRST_LETTER
+ * 
+ * @warning The difficulty comes when we have to write the first syllable text, without the first letter.
+ * The behaviour can have some bugs is this case if the first syllable has some complex styles. It would be a bit stupid to do such a thing, but users are usually very creative when it comes to inventing twisted things...
+ */
 void
 gregorio_write_text (char type, gregorio_character * current_character,
 		     FILE * f, void (*printverb) (FILE *, grewchar *),
@@ -170,6 +164,10 @@ gregorio_write_text (char type, gregorio_character * current_character,
 			    current_character->next_character;
 			}
 		    }
+		  else
+		    {
+		      begin (f, ST_INITIAL);
+		    }
 		  break;
 		default:
 		  begin (f, current_character->cos.s.style);
@@ -318,6 +316,63 @@ gregorio_first_letter (gregorio_score * score)
   return L'\0';
 }
 
+/// the variable that will decide if we're writing old or modern style \n
+/// It is set by gregorio_set_tex_write() and used in gregorio_write_one_tex_char()
+unsigned char tex_write = WRITE_UTF_TEX;
+
+void
+gregorio_set_tex_write(unsigned char new)
+{
+    if (new == WRITE_OLD_TEX)
+      {
+        tex_write = WRITE_OLD_TEX;
+      }
+    else
+      {
+        tex_write = WRITE_UTF_TEX;
+      }
+}
+
+/*!
+ * A function to print one character in TeX. The reason of this function is that
+ * sometimes you need gregorio to write \char 232 (because of some bugs) and
+ *sometimes é, directly in utf8.
+ *
+ */
+void
+gregorio_write_one_tex_char (FILE * f, grewchar to_print)
+{
+    if (tex_write == WRITE_OLD_TEX) 
+      {
+        gregorio_write_one_tex_char_old (f, to_print);
+      }
+    else
+      {
+        gregorio_write_one_tex_char_utf (f, to_print);
+      } 
+}
+
+// the function uses one of the following functions:
+
+void
+gregorio_write_one_tex_char_old (FILE * f, grewchar to_print)
+{
+      if (to_print < 128)
+    {
+      gregorio_print_unichar (f, to_print);
+      return;
+    }
+    fprintf(f, "\\char %d", to_print);
+}
+
+void
+gregorio_write_one_tex_char_utf (FILE * f, grewchar to_print)
+{
+    gregorio_print_unichar (f, to_print);
+}
+
+// this function sets the pointer
+
 /* Here starts the code of the handling of text and styles.
 
 This part is not the easiest, in fact is is the most complicated. The reason is that I want to make something coherent in memory (easy to interprete), and to let the user type whatever he wants. This part was originally written for gabc, that's why it's always talking about it. But it could as well be used by other read plugins, to put things in order.
@@ -343,11 +398,12 @@ The push function pushes a style in the stack, and updates first_style to this e
 void
 gregorio_style_push (det_style ** current_style, unsigned char style)
 {
+  det_style *element;
   if (!current_style)
     {
       return;
     }
-  det_style *element = (det_style *) malloc (sizeof (det_style));
+  element = (det_style *) malloc (sizeof (det_style));
   element->style = style;
   element->previous_style = NULL;
   element->next_style = (*current_style);
@@ -395,11 +451,12 @@ free_styles just free the stack. You may notice that it will never be used in a
 void
 gregorio_free_styles (det_style ** first_style)
 {
+  det_style *current_style;
   if (!first_style)
     {
       return;
     }
-  det_style *current_style = (*first_style);
+  current_style = (*first_style);
   while (current_style)
     {
       current_style = current_style->next_style;
@@ -567,6 +624,24 @@ gregorio_insert_style_after (unsigned char type, unsigned char style,
   (*current_character) = element;
 }
 
+void
+gregorio_insert_char_after (grewchar c,
+			     gregorio_character ** current_character)
+{
+  gregorio_character *element =
+    (gregorio_character *) malloc (sizeof (gregorio_character));
+  element->is_character = 1;
+  element->cos.character = c;
+  element->next_character = (*current_character)->next_character;
+  if ((*current_character)->next_character)
+    {
+      (*current_character)->next_character->previous_character = element;
+    }
+  element->previous_character = (*current_character);
+  (*current_character)->next_character = element;
+  (*current_character) = element;
+}
+
 /*
 
 This function suppresses the corresponding character, updates the double chained list, but does not touch to current_character.
@@ -602,11 +677,12 @@ This function suppresses the current character, updates the double chained list,
 void
 gregorio_suppress_current_character (gregorio_character ** current_character)
 {
+  gregorio_character *thischaracter;
   if (!current_character || !*current_character)
     {
       return;
     }
-  gregorio_character *thischaracter = *current_character;
+  thischaracter = *current_character;
   if ((*current_character)->previous_character)
     {
       (*current_character)->previous_character->next_character =
@@ -629,6 +705,8 @@ current character is a pointer to a gregorio_character. The gregorio_character d
 
 center_is_determined has the values present in characters.h.
 
+Another difficulty is the fact that we must consider characters in verbatim and special character styles like only one block, we can't say the center is in the middle of a verbatim block.
+
 */
 
 void
@@ -667,6 +745,7 @@ gregorio_rebuild_characters (gregorio_character ** param_character,
 	      && gregorio_is_vowel (current_character->cos.character))
 	    {
 	      if (current_character->cos.character == L'i'
+		  || current_character->cos.character == L'I'
 		  || current_character->cos.character == L'u')
 		{
 		  // did you really think it would be that easy?... we have to deal with iota and digamma, that are not aligned the same way... So if the current character is i or u, we check if the next character is also a vowel or not. If it is the case we just pass, else we start the center there.
@@ -696,23 +775,21 @@ gregorio_rebuild_characters (gregorio_character ** param_character,
 		  // remember? this macro has a continue; in it
 		  end_c ();
 		}
-	      begin_center (center_type)
-		center_is_determined = CENTER_DETERMINING_MIDDLE;
+	      begin_center (center_type);
+	      center_is_determined = CENTER_DETERMINING_MIDDLE;
 	      end_c ();
 	    }
 	  // the case where the user has not determined the middle and we are in the middle section of the syllable, but there we encounter something that is not a vowel, so the center ends there.
 	  if (center_is_determined == CENTER_DETERMINING_MIDDLE
 	      && !gregorio_is_vowel (current_character->cos.character))
 	    {
-	      end_center (center_type)
-		center_is_determined = CENTER_FULLY_DETERMINED;
+	      end_center (center_type);
+	      center_is_determined = CENTER_FULLY_DETERMINED;
 	    }
 	  // in the case where it is just a normal character... we simply pass.
 	  end_c ();
 	}
-
 // there starts the second part of the function that deals with the styles characters
-
       if (current_character->cos.s.type == ST_T_BEGIN
 	  && current_character->cos.s.style != ST_CENTER
 	  && current_character->cos.s.style != ST_FORCED_CENTER)
@@ -728,10 +805,35 @@ gregorio_rebuild_characters (gregorio_character ** param_character,
 	      current_style = first_style;
 	      suppress_char_and_end_c ();
 	    }
+	  // if we are determining the end of the middle and we have a VERBATIM or SPECIAL_CHAR style, we end the center determination
+	  if ((current_character->cos.s.style == ST_VERBATIM
+	      || current_character->cos.s.style == ST_SPECIAL_CHAR)
+	      && center_is_determined == CENTER_DETERMINING_MIDDLE)
+	    {
+	      end_center (center_type);
+	      center_is_determined = CENTER_FULLY_DETERMINED;
+	    }
 	  // if it is something to add then we just push the style in the stack and continue.
 	  gregorio_style_push (&first_style, current_character->cos.s.style);
 	  current_style = first_style;
-	  end_c ();
+	  // Here we pass all the characters after a verbatim (or special char) beginning, until we find a style (begin or end)
+	  if (current_character->cos.s.style == ST_VERBATIM
+	      || current_character->cos.s.style == ST_SPECIAL_CHAR)
+	    {
+	      if (current_character->next_character)
+	        {
+	          current_character = current_character->next_character;
+	        }
+	      while (current_character->next_character
+		     && current_character->is_character)
+		{
+		  current_character = current_character->next_character;
+		}
+	    }
+	  else
+	    {
+	      end_c ();
+	    }
 	}
       // if it is a beginning of a center, we call the good macro and end.
       if (current_character->cos.s.type == ST_T_BEGIN
@@ -751,7 +853,8 @@ gregorio_rebuild_characters (gregorio_character ** param_character,
 	      end_c ();
 	    }
 	  //center_is_determined = DETERMINING_MIDDLE; // TODO: not really sure, but shouldn't be there
-	begin_center (center_type) end_c ()}
+	  begin_center (center_type) end_c ();
+	}
       if (current_character->cos.s.type == ST_T_END
 	  && current_character->cos.s.style != ST_CENTER
 	  && current_character->cos.s.style != ST_FORCED_CENTER)
@@ -845,22 +948,22 @@ gregorio_rebuild_characters (gregorio_character ** param_character,
   gregorio_free_styles (&first_style);
 }
 
-/*
-
-This function will determine the behaviour of gregorio when it comes to the recognition of the initial. Basically it will take a gregorio_character list and return the same list, but with a style added : ST_INITIAL. This style will incidate the initial. The center will be placed to the second syllable except if there is a FORCED_CENTER somewhere. Finally all will be xml-compliant and tex-compliant when we call gregorio_rebuild_characters.
-
-If we note <> for the initial and {} for the center, here is what we want:
-
-Po -> <P>{o}
-{A}b -> <>{A}b
-{}a -> <>{}a
-Glo -> <G>{l}o
-Gl{o} -> <G>l{o}
-
-param_character is a pointer to the (pointer to the) first character, it will be modified so that it points to the new first character.
-
-*/
-
+/**
+ * @brief This function will determine the behaviour of gregorio when it comes to the recognition of the initial. 
+ * 
+ * Basically it will take a gregorio_character list and return the same list, but with a style added : ST_INITIAL. This style will incidate the initial. The center will be placed at the second letter, unless there is a FORCED_CENTER somewhere. 
+ * Finally all will be xml-compliant and tex-compliant when we call gregorio_rebuild_characters.
+ *
+ * If we note <> for the initial and {} for the center, here is what we want:
+ *
+ * @li \verbatim Po -> <P>{o} \endverbatim
+ * @li \verbatim {A}b -> <>{A}b \endverbatim
+ * @li \verbatim {}a -> <>{}a \endverbatim
+ * @li \verbatim Glo -> <G>{l}o \endverbatim
+ * @li \verbatim Gl{o} -> <G>l{o} \endverbatim
+ *
+ * @param param_character is a pointer to the (pointer to the) first character, it will be modified so that it points to the new first character.
+ */
 void
 gregorio_rebuild_first_syllable (gregorio_character ** param_character)
 {
@@ -888,11 +991,12 @@ gregorio_rebuild_first_syllable (gregorio_character ** param_character)
 	    }
 	  if (current_character->cos.s.style == ST_CENTER)
 	    {
-		  // we have to do it in case param_character (the first character) is a ST_CENTER beginning
-		  if (!current_character->previous_character && current_character == *param_character)
-		    {
-			  *param_character = (*param_character)->next_character;
-			}
+	      // we have to do it in case param_character (the first character) is a ST_CENTER beginning
+	      if (!current_character->previous_character
+		  && current_character == *param_character)
+		{
+		  *param_character = (*param_character)->next_character;
+		}
 	      gregorio_suppress_current_character (&current_character);
 	      continue;
 	    }
@@ -908,7 +1012,7 @@ gregorio_rebuild_first_syllable (gregorio_character ** param_character)
 	  && current_character->cos.s.style == ST_FORCED_CENTER
 	  && letter == 0)
 	{
-	  // we don't touche anything after a FORCED_CENTER, so we put an empty INITIAL style just before
+	  // we don't touch anything after a FORCED_CENTER, so we put an empty INITIAL style just before
 	  gregorio_insert_style_before (ST_T_BEGIN, ST_INITIAL,
 					current_character);
 	  current_character = current_character->previous_character;
@@ -916,6 +1020,48 @@ gregorio_rebuild_first_syllable (gregorio_character ** param_character)
 				       &current_character);
 	  break;
 	}
+	// this if is a hack to make gregorio consider verbatim blocks and special chars like one block, not a sequence of letters
+	  if (!current_character->is_character
+	  && current_character->cos.s.type == ST_T_BEGIN
+	  && (current_character->cos.s.style == ST_VERBATIM
+	  || current_character->cos.s.style == ST_SPECIAL_CHAR))
+	  
+	{
+      if (letter == 0)
+	{
+	  letter = 1;
+	  gregorio_insert_style_before (ST_T_BEGIN, ST_INITIAL,
+					current_character);
+	  if (current_character->next_character)
+	    {
+	      current_character = current_character->next_character;
+	    }
+	  while(current_character->next_character && current_character->is_character)
+	    {
+	      current_character = current_character->next_character;
+	    }
+	  gregorio_insert_style_after (ST_T_END, ST_INITIAL,
+				       &current_character);
+	  continue;
+	}
+      if (letter == 1 && forced_center == 0)
+	{
+	  gregorio_insert_style_before (ST_T_BEGIN, ST_CENTER,
+					current_character);
+	  if (current_character->next_character)
+	    {
+	      current_character = current_character->next_character;
+	    }
+	  while(current_character->next_character && current_character->is_character)
+	    {
+	      current_character = current_character->next_character;
+	    }
+	  gregorio_insert_style_after (ST_T_END, ST_CENTER,
+				       &current_character);
+	  // and we're done
+	  break;
+	}
+	}
       if (current_character->is_character && letter == 0)
 	{
 	  letter = 1;
@@ -936,12 +1082,14 @@ gregorio_rebuild_first_syllable (gregorio_character ** param_character)
 	  // and we're done
 	  break;
 	}
-	  // case where there is only one letter
-      if (!current_character->next_character
-	  && forced_center == 0)
+      // case where there is only one letter
+      if (!current_character->next_character && forced_center == 0)
 	{
 	  gregorio_insert_style_after (ST_T_BEGIN, ST_CENTER,
-					&current_character);
+				       &current_character);
+	  // not such a good idea: some people might want nothing here
+	  //gregorio_insert_char_after (L'-',
+		//		       &current_character);
 	  gregorio_insert_style_after (ST_T_END, ST_CENTER,
 				       &current_character);
 	  // and we're done
