@@ -1,6 +1,6 @@
-/* 
+/*
 Gregorio dump output format.
-Copyright (C) 2007 Elie Roux <elie.roux@enst-bretagne.fr>
+Copyright (C) 2007-2009 Elie Roux <elie.roux@telecom-bretagne.eu>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,16 +19,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include "gettext.h"
-#define _(str) gettext(str)
-#define N_(str) str
 #include <gregorio/struct.h>
 #include <gregorio/unicode.h>
-#include <gregorio/plugin.h>
+#if ALL_STATIC == 0
+    #include <gregorio/plugin.h>
+#endif
 #include <gregorio/messages.h>
 
 #include "dump.h"
 
+#if ALL_STATIC == 0
 DECLARE_PLUGIN(dump)
 {
   .id = "dump",
@@ -42,9 +42,15 @@ DECLARE_PLUGIN(dump)
 
   .write = write_score
 };
+#endif
 
+#if ALL_STATIC == 0
 void
 write_score (FILE * f, gregorio_score * score)
+#else
+void
+dump_write_score (FILE * f, gregorio_score * score)
+#endif
 {
   gregorio_syllable *syllable = score->first_syllable;
   gregorio_voice_info *voice_info = score->first_voice_info;
@@ -52,14 +58,14 @@ write_score (FILE * f, gregorio_score * score)
   gregorio_element *element;
   gregorio_glyph *glyph;
   gregorio_note *note;
+  int annotation_num;
 
   if (!f) {
       gregorio_message (_
 			   ("call with NULL file"),
-			   "libgregorio_gregoriotex_write_score", ERROR, 0);
+			   "gregoriotex_write_score", ERROR, 0);
 			   return;
   }
-
   fprintf (f,
 	   "=====================================================================\n SCORE INFOS\n=====================================================================\n");
   if (score->number_of_voices)
@@ -71,17 +77,72 @@ write_score (FILE * f, gregorio_score * score)
     {
       fprintf (f, "   name                      %s\n", score->name);
     }
+  if (score->gabc_copyright)
+    {
+      fprintf (f, "   gabc_copyright            %s\n", score->gabc_copyright);
+    }
+  if (score->score_copyright)
+    {
+      fprintf (f, "   score_copyright           %s\n", score->score_copyright);
+    }
   if (score->office_part)
     {
       fprintf (f, "   office_part               %s\n", score->office_part);
     }
-  if (score->initial_style)
+  if (score->occasion)
     {
-      fprintf (f, "   initial_style             %d\n", score->initial_style);
+      fprintf (f, "   occasion                  %s\n", score->occasion);
     }
-  if (score->mode)
+  if (score->meter)
     {
-      fprintf (f, "   mode                      %d\n", score->mode);
+      fprintf (f, "   meter                     %s\n", score->meter);
+    }
+  if (score->commentary)
+    {
+      fprintf (f, "   commentary                %s\n", score->commentary);
+    }
+  if (score->arranger)
+    {
+      fprintf (f, "   arranger                  %s\n", score->arranger);
+    }
+  if (score->si.author)
+    {
+      fprintf (f, "   author                    %s\n",
+	       score->si.author);
+    }
+  if (score->si.date)
+    {
+      fprintf (f, "   date                      %s\n", score->si.date);
+    }
+  if (score->si.manuscript)
+    {
+      fprintf (f, "   manuscript                %s\n",
+	       score->si.manuscript);
+    }
+  if (score->si.manuscript_reference)
+    {
+      fprintf (f, "   manuscript_reference      %s\n",
+	       score->si.manuscript_reference);
+    }
+  if (score->si.manuscript_storage_place)
+    {
+      fprintf (f, "   manuscript_storage_place  %s\n",
+	       score->si.manuscript_storage_place);
+    }
+  if (score->si.book)
+    {
+      fprintf (f, "   book                      %s\n",
+	       score->si.book);
+    }
+  if (score->si.transcriber)
+    {
+      fprintf (f, "   transcriber               %s\n",
+	       score->si.transcriber);
+    }
+  if (score->si.transcription_date)
+    {
+      fprintf (f, "   transcription_date        %s\n",
+	       score->si.transcription_date);
     }
   if (score->lilypond_preamble)
     {
@@ -103,6 +164,18 @@ write_score (FILE * f, gregorio_score * score)
       fprintf (f, "   gregoriotex_font          %s\n",
 	       score->gregoriotex_font);
     }
+  if (score->mode)
+    {
+      fprintf (f, "   mode                      %d\n", score->mode);
+    }
+  if (score->initial_style)
+    {
+      fprintf (f, "   initial_style             %d\n", score->initial_style);
+    }
+  if (score->user_notes)
+    {
+      fprintf (f, "   user_notes                %s\n", score->user_notes);
+    }
   fprintf (f,
 	   "\n\n=====================================================================\n VOICES INFOS\n=====================================================================\n");
   for (i = 0; i < score->number_of_voices; i++)
@@ -112,46 +185,19 @@ write_score (FILE * f, gregorio_score * score)
 	{
 	  fprintf (f, "   initial_key               %d (%s)\n",
 		   voice_info->initial_key,
-		   libgregorio_dump_key_to_char (voice_info->initial_key));
+		   dump_key_to_char (voice_info->initial_key));
+      if (voice_info->flatted_key == FLAT_KEY)
+        {
+           fprintf (f, "   flat_key                  FLAT_KEY\n");
+        }
 	}
-      if (voice_info->anotation)
+      for (annotation_num = 0; annotation_num < NUM_ANNOTATIONS; ++annotation_num)
 	{
-	  fprintf (f, "   anotation                 %s\n",
-		   voice_info->anotation);
+	  if (voice_info->annotation [annotation_num])
+	{
+	      fprintf (f, "   annotation                %s\n",
+		       voice_info->annotation [annotation_num]);
 	}
-      if (voice_info->author)
-	{
-	  fprintf (f, "   author                    %s\n",
-		   voice_info->author);
-	}
-      if (voice_info->date)
-	{
-	  fprintf (f, "   date                      %s\n", voice_info->date);
-	}
-      if (voice_info->manuscript)
-	{
-	  fprintf (f, "   manuscript                %s\n",
-		   voice_info->manuscript);
-	}
-      if (voice_info->reference)
-	{
-	  fprintf (f, "   reference                 %s\n",
-		   voice_info->reference);
-	}
-      if (voice_info->storage_place)
-	{
-	  fprintf (f, "   storage_place             %s\n",
-		   voice_info->storage_place);
-	}
-      if (voice_info->translator)
-	{
-	  fprintf (f, "   translator                %s\n",
-		   voice_info->translator);
-	}
-      if (voice_info->translation_date)
-	{
-	  fprintf (f, "   translation_date          %s\n",
-		   voice_info->translation_date);
 	}
       if (voice_info->style)
 	{
@@ -171,13 +217,18 @@ write_score (FILE * f, gregorio_score * score)
       if (syllable->type)
 	{
 	  fprintf (f, "   type                      %d (%s)\n",
-		   syllable->type, libgregorio_dump_type (syllable->type));
+		   syllable->type, dump_type (syllable->type));
 	}
       if (syllable->position)
 	{
 	  fprintf (f, "   position                  %d (%s)\n",
 		   syllable->position,
-		   libgregorio_dump_syllable_position (syllable->position));
+		   dump_syllable_position (syllable->position));
+	}
+	  if (syllable->additional_infos)
+	{
+	  fprintf (f, "   additional infos                   %s\n",
+	  dump_rare_sign(syllable->additional_infos));
 	}
       if (syllable->text)
 	{
@@ -185,12 +236,16 @@ write_score (FILE * f, gregorio_score * score)
 	    {
 	      fprintf (f, "\n  Text\n");
 	    }
-	  libgregorio_dump_write_characters (f, syllable->text);
+	  dump_write_characters (f, syllable->text);
 	}
       if (syllable->translation)
 	{
 	  fprintf (f, "\n  Translation\n");
-	  libgregorio_dump_write_characters (f, syllable->translation);
+	  dump_write_characters (f, syllable->translation);
+	}
+      if (syllable->abovelinestext)
+	{
+	  fprintf (f, "\n  Abovelinestext\n    %s", syllable->abovelinestext);
 	}
       element = syllable->elements[0];
       while (element)
@@ -200,13 +255,13 @@ write_score (FILE * f, gregorio_score * score)
 	  if (element->type)
 	    {
 	      fprintf (f, "     type                    %d (%s)\n",
-		       element->type, libgregorio_dump_type (element->type));
+		       element->type, dump_type (element->type));
 	    }
 	  if (element->element_type && element->type == GRE_ELEMENT)
 	    {
 	      fprintf (f, "     element_type            %d (%s)\n",
 		       element->element_type,
-		       libgregorio_dump_element_type (element->element_type));
+		       dump_element_type (element->element_type));
 	    }
 	  if (element->element_type && element->type == GRE_CUSTO)
 	    {
@@ -217,23 +272,52 @@ write_score (FILE * f, gregorio_score * score)
 	    {
 	      fprintf (f, "     element_type            %d (%s)\n",
 		       element->element_type,
-		       libgregorio_dump_space_type (element->element_type));
+		       dump_space_type (element->element_type));
+	    }
+	  if (element->type == GRE_TEXVERB_ELEMENT)
+	    {
+          fprintf (f, "     TeX string              \"%s\"\n",
+		       element->texverb);
+	    }
+	  if (element->type == GRE_ALT)
+	    {
+          fprintf (f, "     Above lines text        \"%s\"\n",
+		       element->texverb);
 	    }
 	  if (element->element_type && element->type == GRE_BAR)
 	    {
 	      fprintf (f, "     element_type            %d (%s)\n",
 		       element->element_type,
-		       libgregorio_dump_bar_type (element->element_type));
+           dump_bar_type (element->element_type));
+		  if (element->additional_infos)
+		    {
+	          fprintf (f, "     additional_infos        %d (%s)\n",
+		        element->element_type,
+		        dump_rare_sign (element->additional_infos));
+		    }
 	    }
 	  if (element->element_type && element->type == GRE_C_KEY_CHANGE)
 	    {
 	      fprintf (f, "     element_type            %d (c%d)\n",
 		       element->element_type, element->element_type - 48);
+          if (element->additional_infos == FLAT_KEY)
+            {
+              fprintf(f, "     additional_infos        FLAT_KEY\n");       
+            }
 	    }
 	  if (element->element_type && element->type == GRE_F_KEY_CHANGE)
 	    {
 	      fprintf (f, "     element_type            %d (f%d)\n",
 		       element->element_type, element->element_type - 48);
+          if (element->additional_infos == FLAT_KEY)
+            {
+              fprintf(f, "     additional_infos        FLAT_KEY\n");        
+            }
+	    }
+	  if (element->element_type && element->type == GRE_END_OF_LINE)
+	    {
+	      fprintf (f, "     element_type            %d (%s)\n",
+		       element->element_type, dump_type (element->element_type));
 	    }
 	  glyph = element->first_glyph;
 	  while (glyph)
@@ -243,28 +327,33 @@ write_score (FILE * f, gregorio_score * score)
 	      if (glyph->type)
 		{
 		  fprintf (f, "       type                  %d (%s)\n",
-			   glyph->type, libgregorio_dump_type (glyph->type));
+			   glyph->type, dump_type (glyph->type));
 		}
+          if (glyph->type == GRE_TEXVERB_GLYPH)
+	    {
+          fprintf (f, "       TeX string            \"%s\"\n",
+		     glyph->texverb);
+	    }
 	      if (glyph->glyph_type)
 		{
 		  if (glyph->type == GRE_SPACE)
 		    {
 		      fprintf (f, "       glyph_type            %d (%s)\n",
 			       glyph->glyph_type,
-			       libgregorio_dump_space_type (glyph->
+			       dump_space_type (glyph->
 							    glyph_type));
 		    }
 		  if (glyph->type == GRE_BAR)
 		    {
 		      fprintf (f, "       glyph_type            %d (%s)\n",
 			       glyph->glyph_type,
-			       libgregorio_dump_bar_type (glyph->glyph_type));
+			       dump_bar_type (glyph->glyph_type));
 		    }
 		  if ((glyph->type == GRE_FLAT)
 		      || (glyph->type == GRE_NATURAL))
 		    {
-		      fprintf (f, "       glyph_type            %d\n",
-			       glyph->glyph_type);
+		      fprintf (f, "       glyph_type            %d (%c)\n",
+			       glyph->glyph_type, glyph->glyph_type);
 		    }
 		  if ((glyph->type != GRE_SPACE) && (glyph->type != GRE_BAR)
 		      && (glyph->type != GRE_FLAT)
@@ -272,15 +361,22 @@ write_score (FILE * f, gregorio_score * score)
 		    {
 		      fprintf (f, "       glyph_type            %d (%s)\n",
 			       glyph->glyph_type,
-			       libgregorio_dump_glyph_type (glyph->
+			       dump_glyph_type (glyph->
 							    glyph_type));
 		    }
 		}
-	      if (glyph->liquescentia)
+	      if (glyph->liquescentia && glyph->glyph_type != GRE_BAR)
 		{
 		  fprintf (f, "       liquescentia          %d (%s)\n",
 			   glyph->liquescentia,
-			   libgregorio_dump_liquescentia (glyph->
+			   dump_liquescentia (glyph->
+							  liquescentia));
+		}
+	      if (glyph->liquescentia && glyph->glyph_type == GRE_BAR)
+		{
+		  fprintf (f, "       liquescentia          %d (%s)\n",
+			   glyph->liquescentia,
+			   dump_rare_sign (glyph->
 							  liquescentia));
 		}
 	      note = glyph->first_note;
@@ -292,55 +388,69 @@ write_score (FILE * f, gregorio_score * score)
 		    {
 		      fprintf (f, "         type                %d (%s)\n",
 			       note->type,
-			       libgregorio_dump_type (note->type));
+			       dump_type (note->type));
 		    }
 		  if (note->pitch)
 		    {
 		      fprintf (f, "         pitch               %c\n",
 			       note->pitch);
 		    }
+		  if (note->texverb)
+		    {
+		      fprintf (f, "         TeX string          \"%s\"\n",
+		       note->texverb);
+		    }
+		  if (note->choral_sign)
+		    {
+		      fprintf (f, "         Choral Sign         \"%s\"\n",
+		       note->choral_sign);
+		    }
 		  if (note->shape)
 		    {
 		      fprintf (f, "         shape               %d (%s)\n",
 			       note->shape,
-			       libgregorio_dump_shape (note->shape));
+			       dump_shape (note->shape));
 		    }
 		  if (note->signs)
 		    {
 		      fprintf (f, "         signs               %d (%s)\n",
 			       note->signs,
-			       libgregorio_dump_signs (note->signs));
+			       dump_signs (note->signs));
 		    }
 		  if (note->rare_sign)
 		    {
 		      fprintf (f, "         rare sign           %d (%s)\n",
 			       note->rare_sign,
-			       libgregorio_dump_rare_sign (note->rare_sign));
+			       dump_rare_sign (note->rare_sign));
 		    }
 		  if (note->liquescentia)
 		    {
 		      fprintf (f, "         liquescentia        %d (%s)\n",
 			       note->liquescentia,
-			       libgregorio_dump_liquescentia (note->
+			       dump_liquescentia (note->
 							      liquescentia));
 		    }
 		  if (note->h_episemus_type)
 		    {
-		      fprintf (f, "         h_episemus_type     %d (%s)\n",
-			       note->h_episemus_type,
-			       libgregorio_dump_h_episemus_type (note->
-								 h_episemus_type));
+		      fprintf (f, "         h_episemus_type     %d (",
+			       note->h_episemus_type);
+			    fprintf(f, "%s", dump_h_episemus_type (simple_htype(note-> h_episemus_type)));
+			    if (has_bottom(note->h_episemus_type))
+			      {
+              fprintf(f, " & H_BOTTOM");
+			      }
+			    fprintf(f, ")\n");
 		    }
 		  if (note->h_episemus_top_note)
 		    {
-		      fprintf (f, "         h_episemus_top_note %d\n",
+		      fprintf (f, "         h_episemus_top_note %c\n",
 			       note->h_episemus_top_note);
 		    }
-		  note = note->next_note;
+		  note = note->next;
 		}
-	      glyph = glyph->next_glyph;
+	      glyph = glyph->next;
 	    }
-	  element = element->next_element;
+	  element = element->next;
 	}
       fprintf (f,
 	       "=====================================================================\n");
@@ -349,7 +459,7 @@ write_score (FILE * f, gregorio_score * score)
 }
 
 const char *
-libgregorio_dump_style_to_string (unsigned char style)
+dump_style_to_string (unsigned char style)
 {
   switch (style)
     {
@@ -374,6 +484,9 @@ libgregorio_dump_style_to_string (unsigned char style)
     case ST_TT:
       return "           ST_TT";
       break;
+    case ST_UNDERLINED:
+      return "   ST_UNDERLINED";
+      break;
     case ST_SMALL_CAPS:
       return "   ST_SMALL_CAPS";
       break;
@@ -390,7 +503,7 @@ libgregorio_dump_style_to_string (unsigned char style)
 }
 
 void
-libgregorio_dump_write_characters (FILE * f,
+dump_write_characters (FILE * f,
 				   gregorio_character * current_character)
 {
   while (current_character)
@@ -399,21 +512,22 @@ libgregorio_dump_write_characters (FILE * f,
 	       "---------------------------------------------------------------------\n");
       if (current_character->is_character)
 	{
-	  fprintf (f, "     character                 %lc\n",
-		   current_character->cos.character);
+	  fprintf (f, "     character                 ");
+	  gregorio_print_unichar(f, current_character->cos.character);
+	  fprintf (f, "\n");
 	}
       else
 	{
 	  if (current_character->cos.s.type == ST_T_BEGIN)
 	    {
 	      fprintf (f, "     beginning of style   %s\n",
-		       libgregorio_dump_style_to_string (current_character->
+		       dump_style_to_string (current_character->
 							 cos.s.style));
 	    }
 	  else
 	    {
 	      fprintf (f, "     end of style         %s\n",
-		       libgregorio_dump_style_to_string (current_character->
+		       dump_style_to_string (current_character->
 							 cos.s.style));
 	    }
 	}
@@ -422,7 +536,7 @@ libgregorio_dump_write_characters (FILE * f,
 }
 
 const char *
-libgregorio_dump_key_to_char (int key)
+dump_key_to_char (int key)
 {
   const char *str;
   switch (key)
@@ -460,7 +574,7 @@ libgregorio_dump_key_to_char (int key)
 
 
 const char *
-libgregorio_dump_syllable_position (char pos)
+dump_syllable_position (char pos)
 {
   const char *str;
   switch (pos)
@@ -486,7 +600,7 @@ libgregorio_dump_syllable_position (char pos)
 
 
 const char *
-libgregorio_dump_type (char type)
+dump_type (char type)
 {
   const char *str;
   switch (type)
@@ -503,6 +617,9 @@ libgregorio_dump_type (char type)
     case GRE_FLAT:
       str = "GRE_FLAT";
       break;
+    case GRE_SHARP:
+      str = "GRE_SHARP";
+      break;
     case GRE_NATURAL:
       str = "GRE_NATURAL";
       break;
@@ -515,6 +632,9 @@ libgregorio_dump_type (char type)
     case GRE_END_OF_LINE:
       str = "GRE_END_OF_LINE";
       break;
+    case GRE_END_OF_PAR:
+      str = "GRE_END_OF_PAR";
+      break;
     case GRE_CUSTO:
       str = "GRE_CUSTO";
       break;
@@ -527,6 +647,12 @@ libgregorio_dump_type (char type)
     case GRE_SYLLABLE:
       str = "GRE_SYLLABLE";
       break;
+    case GRE_TEXVERB_GLYPH:
+      str = "GRE_TEXVERB_GLYPH";
+      break;
+    case GRE_TEXVERB_ELEMENT:
+      str = "GRE_TEXVERB_ELEMENT";
+      break;
     default:
       str = "unknown";
       break;
@@ -536,7 +662,7 @@ libgregorio_dump_type (char type)
 
 
 const char *
-libgregorio_dump_bar_type (char element_type)
+dump_bar_type (char element_type)
 {
   const char *str;
   switch (element_type)
@@ -559,6 +685,24 @@ libgregorio_dump_bar_type (char element_type)
     case B_DIVISIO_FINALIS:
       str = "B_DIVISIO_FINALIS";
       break;
+    case B_DIVISIO_MINOR_D1:
+      str = "B_DIVISIO_MINOR_D1";
+      break;
+    case B_DIVISIO_MINOR_D2:
+      str = "B_DIVISIO_MINOR_D2";
+      break;
+    case B_DIVISIO_MINOR_D3:
+      str = "B_DIVISIO_MINOR_D3";
+      break;
+    case B_DIVISIO_MINOR_D4:
+      str = "B_DIVISIO_MINOR_D4";
+      break;
+    case B_DIVISIO_MINOR_D5:
+      str = "B_DIVISIO_MINOR_D5";
+      break;
+    case B_DIVISIO_MINOR_D6:
+      str = "B_DIVISIO_MINOR_D6";
+      break;
     default:
       str = "unknown";
       break;
@@ -568,7 +712,7 @@ libgregorio_dump_bar_type (char element_type)
 
 
 const char *
-libgregorio_dump_space_type (char element_type)
+dump_space_type (char element_type)
 {
   const char *str;
   switch (element_type)
@@ -591,6 +735,15 @@ libgregorio_dump_space_type (char element_type)
     case SP_GLYPH_SPACE:
       str = "SP_GLYPH_SPACE";
       break;
+    case SP_GLYPH_SPACE_NB:
+      str = "SP_GLYPH_SPACE_NB";
+      break;
+    case SP_LARGER_SPACE_NB:
+      str = "SP_LARGER_SPACE_NB";
+      break;
+    case SP_NEUMATIC_CUT_NB:
+      str = "SP_NEUMATIC_CUT_NB";
+      break;
     default:
       str = "unknown";
       break;
@@ -601,7 +754,7 @@ libgregorio_dump_space_type (char element_type)
 
 // not so sure it has still a meaning now...
 const char *
-libgregorio_dump_element_type (char element_type)
+dump_element_type (char element_type)
 {
   const char *str;
   switch (element_type)
@@ -644,7 +797,7 @@ libgregorio_dump_element_type (char element_type)
 }
 
 const char *
-libgregorio_dump_liquescentia (char liquescentia)
+dump_liquescentia (char liquescentia)
 {
   const char *str;
   switch (liquescentia)
@@ -686,7 +839,7 @@ libgregorio_dump_liquescentia (char liquescentia)
   return str;
 }
 const char *
-libgregorio_dump_glyph_type (char glyph_type)
+dump_glyph_type (char glyph_type)
 {
   const char *str;
   switch (glyph_type)
@@ -798,7 +951,7 @@ libgregorio_dump_glyph_type (char glyph_type)
 }
 
 const char *
-libgregorio_dump_shape (char shape)
+dump_shape (char shape)
 {
   const char *str;
   switch (shape)
@@ -836,6 +989,9 @@ libgregorio_dump_shape (char shape)
     case S_ORISCUS_AUCTUS:
       str = "S_ORISCUS_AUCTUS";
       break;
+    case S_ORISCUS_DEMINUTUS:
+      str = "S_ORISCUS_DEMINUTUS";
+      break;
     case S_QUILISMA:
       str = "S_QUILISMA";
       break;
@@ -862,13 +1018,16 @@ libgregorio_dump_shape (char shape)
       break;
     case S_PUNCTUM_CAVUM:
       str = "S_PUNCTUM_CAVUM";
-      break; 
+      break;
     case S_LINEA_PUNCTUM:
       str = "S_LINEA_PUNCTUM";
       break;
     case S_LINEA_PUNCTUM_CAVUM:
       str = "S_LINEA_PUNCTUM_CAVUM";
-      break;      
+      break;
+    case S_LINEA:
+      str = "S_LINEA";
+      break;
     default:
       str = "unknown";
       break;
@@ -877,7 +1036,7 @@ libgregorio_dump_shape (char shape)
 }
 
 const char *
-libgregorio_dump_signs (char signs)
+dump_signs (char signs)
 {
   const char *str;
   switch (signs)
@@ -907,8 +1066,9 @@ libgregorio_dump_signs (char signs)
   return str;
 }
 
+// a function dumping rare_signs or additional_informations
 const char *
-libgregorio_dump_rare_sign (char rare_sign)
+dump_rare_sign (char rare_sign)
 {
   const char *str;
   switch (rare_sign)
@@ -928,6 +1088,39 @@ libgregorio_dump_rare_sign (char rare_sign)
     case _SEMI_CIRCULUS_REVERSUS:
       str = "_SEMI_CIRCULUS_REVERSUS";
       break;
+    case _ICTUS_A:
+      str = "_ICTUS_A";
+      break;
+    case _ICTUS_T:
+      str = "_ICTUS_T";
+      break;
+    case _V_EPISEMUS_ICTUS_A:
+      str = "_V_EPISEMUS_ICTUS_A";
+      break;
+    case _V_EPISEMUS_ICTUS_T:
+      str = "_V_EPISEMUS_ICTUS_T";
+      break;
+    case _V_EPISEMUS:
+      str = "_V_EPISEMUS";
+      break;
+    case _V_EPISEMUS_H_EPISEMUS:
+      str = "_V_EPISEMUS_H_EPISEMUS";
+      break;
+    case _V_EPISEMUS_H_EPISEMUS_ICTUS_A:
+      str = "_V_EPISEMUS_H_EPISEMUS_ICTUS_A";
+      break;
+    case _V_EPISEMUS_H_EPISEMUS_ICTUS_T:
+      str = "_V_EPISEMUS_H_EPISEMUS_ICTUS_T";
+      break;
+    case _H_EPISEMUS_ICTUS_A:
+      str = "_H_EPISEMUS_ICTUS_A";
+      break;
+    case _H_EPISEMUS_ICTUS_T:
+      str = "_H_EPISEMUS_ICTUS_T";
+      break;
+    case _H_EPISEMUS:
+      str = "_H_EPISEMUS";
+      break;
     default:
       str = "unknown";
       break;
@@ -936,7 +1129,7 @@ libgregorio_dump_rare_sign (char rare_sign)
 }
 
 const char *
-libgregorio_dump_h_episemus_type (char h_episemus_type)
+dump_h_episemus_type (unsigned char h_episemus_type)
 {
   const char *str;
   switch (h_episemus_type)
