@@ -1,6 +1,6 @@
 /* 
 Gregorio glyph determination in gabc input.
-Copyright (C) 2006 Elie Roux <elie.roux@enst-bretagne.fr>
+Copyright (C) 2006-2009 Elie Roux <elie.roux@telecom-bretagne.eu>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -31,12 +31,12 @@ close_glyph (gregorio_glyph ** current_glyph, char glyph_type,
 
 
 gregorio_glyph *
-libgregorio_gabc_det_glyphs_from_string (char *str, int *initial_key)
+gabc_det_glyphs_from_string (char *str, int *initial_key, char *macros[10])
 {
   gregorio_note *tmp;
   gregorio_glyph *final;
-  tmp = libgregorio_gabc_det_notes_from_string (str);
-  final = libgregorio_gabc_det_glyphs_from_notes (tmp, initial_key);
+  tmp = gabc_det_notes_from_string (str, macros);
+  final = gabc_det_glyphs_from_notes (tmp, initial_key);
   return final;
 }
 
@@ -44,7 +44,7 @@ libgregorio_gabc_det_glyphs_from_string (char *str, int *initial_key)
 /****************************
  * 
  * First see the comments of
- * libgregorio_gabc_det_glyphs_from_notes. This function is used when
+ * gabc_det_glyphs_from_notes. This function is used when
  * we have finished to determine a glyph. We have the last glyph that
  * have been added: last_glyph. The glyph we want to add is given by
  * glyph_type and liquescentia.
@@ -72,107 +72,131 @@ close_glyph (gregorio_glyph ** last_glyph, char glyph_type,
     }
 
   gregorio_add_glyph (last_glyph, glyph_type, *first_note, liquescentia);
-  if (current_note->next_note)
+  if (current_note->next)
     {
-      current_note->next_note->previous_note = NULL;
-      *first_note = current_note->next_note;
-      current_note->next_note = NULL;
+      current_note->next->previous = NULL;
+      *first_note = current_note->next;
+      current_note->next = NULL;
     }
 
   // here we "patch" the structure for bivirga, tristropha, etc.
   // the idea is not to have a S_BIVIRGA in the shape of the note (which is dirty)
   // but rather a G_BIVIRGA in the glyph (which is the case now) and two virgas
 
-  if (glyph_type == G_BIVIRGA || glyph_type == G_DISTROPHA || glyph_type == G_TRIVIRGA || glyph_type == G_TRISTROPHA || glyph_type == G_DISTROPHA_AUCTA || glyph_type == G_TRISTROPHA_AUCTA)
-   {
-    gregorio_go_to_first_note(&current_note);
-    while (current_note)
-      {
-        if (current_note -> shape == S_BIVIRGA)
-          {
-            gregorio_add_note (&added_notes, current_note -> pitch, S_VIRGA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_VIRGA, current_note -> signs, current_note -> liquescentia, current_note -> h_episemus_type);
-          }
-        if (current_note -> shape == S_TRIVIRGA)
-          {
-            gregorio_add_note (&added_notes, current_note -> pitch, S_VIRGA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_VIRGA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_VIRGA, current_note -> signs, current_note -> liquescentia, current_note -> h_episemus_type);
-          }
-        if (current_note -> shape == S_DISTROPHA)
-          {
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, current_note -> signs, current_note -> liquescentia, current_note -> h_episemus_type);
-          }
-        if (current_note -> shape == S_DISTROPHA_AUCTA)
-          {
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA_AUCTA, current_note -> signs, current_note -> liquescentia, current_note -> h_episemus_type);
-          }
-        if (current_note -> shape == S_TRISTROPHA)
-          {
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, current_note -> signs, current_note -> liquescentia, current_note -> h_episemus_type);
-          }
-        if (current_note -> shape == S_TRISTROPHA_AUCTA)
-          {
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA, _NO_SIGN, L_NO_LIQUESCENTIA, current_note -> h_episemus_type);
-            gregorio_add_note (&added_notes, current_note -> pitch, S_STROPHA_AUCTA, current_note -> signs, current_note -> liquescentia, current_note -> h_episemus_type);
-          }
-        // now we have what we want, we set up the links and free the old note
-        if (current_note->next_note)
-          {
-            current_note -> next_note -> previous_note = added_notes;
-            added_notes -> next_note = current_note -> next_note;
-          }
-        gregorio_go_to_first_note (&added_notes);
-        if (current_note->previous_note)
-          {
-            current_note->previous_note -> next_note = added_notes;
-            added_notes -> previous_note = current_note -> previous_note;
-          }
-        if (!current_note -> previous_note && !current_note -> next_note)
-          {
-            current_note = added_notes;
-            break;
-          }
-        else 
-          {
-            gregorio_free_one_note (&current_note); // automatically sets first_note to the next note
-          }
-      }
-      gregorio_go_to_first_note(&current_note);
+  if (glyph_type == G_BIVIRGA || glyph_type == G_DISTROPHA
+      || glyph_type == G_TRIVIRGA || glyph_type == G_TRISTROPHA
+      || glyph_type == G_DISTROPHA_AUCTA || glyph_type == G_TRISTROPHA_AUCTA)
+    {
+      gregorio_go_to_first_note (&current_note);
+      while (current_note)
+	{
+	  switch (current_note->shape)
+	    {
+	    case S_TRIVIRGA:
+	      gregorio_add_note (&added_notes, current_note->pitch, S_VIRGA,
+				 _NO_SIGN, L_NO_LIQUESCENTIA,
+				 current_note->h_episemus_type);
+	    case S_BIVIRGA:
+	      gregorio_add_note (&added_notes, current_note->pitch, S_VIRGA,
+				 _NO_SIGN, L_NO_LIQUESCENTIA,
+				 current_note->h_episemus_type);
+	      gregorio_add_note (&added_notes, current_note->pitch, S_VIRGA,
+				 current_note->signs,
+				 current_note->liquescentia,
+				 current_note->h_episemus_type);
+	      break;
+	    case S_TRISTROPHA:
+	      gregorio_add_note (&added_notes, current_note->pitch, S_STROPHA,
+				 _NO_SIGN, L_NO_LIQUESCENTIA,
+				 current_note->h_episemus_type);
+	    case S_DISTROPHA:
+	      gregorio_add_note (&added_notes, current_note->pitch, S_STROPHA,
+				 _NO_SIGN, L_NO_LIQUESCENTIA,
+				 current_note->h_episemus_type);
+	      gregorio_add_note (&added_notes, current_note->pitch, S_STROPHA,
+				 current_note->signs,
+				 current_note->liquescentia,
+				 current_note->h_episemus_type);
+	      break;
+	    case S_TRISTROPHA_AUCTA:
+	      gregorio_add_note (&added_notes, current_note->pitch, S_STROPHA,
+				 _NO_SIGN, L_NO_LIQUESCENTIA,
+				 current_note->h_episemus_type);
+	    case S_DISTROPHA_AUCTA:
+	      gregorio_add_note (&added_notes, current_note->pitch, S_STROPHA,
+				 _NO_SIGN, L_NO_LIQUESCENTIA,
+				 current_note->h_episemus_type);
+	      gregorio_add_note (&added_notes, current_note->pitch,
+				 S_STROPHA_AUCTA, current_note->signs,
+				 current_note->liquescentia,
+				 current_note->h_episemus_type);
+	      break;
+	    default:
+	      break;
+	    }
+	  // this is the case of two separate virga that have been spotted as a bivirga
+	  if (!added_notes)
+	    {
+	      break;
+	    }
+	  // now we have what we want, we set up the links and free the old note
+	  if (current_note->next)
+	    {
+	      current_note->next->previous = added_notes;
+	      added_notes->next = current_note->next;
+	    }
+	  gregorio_go_to_first_note (&added_notes);
+	  if (current_note->previous)
+	    {
+	      current_note->previous->next = added_notes;
+	      added_notes->previous = current_note->previous;
+	    }
+	  if (!current_note->previous && !current_note->next)
+	    {
+	      current_note = added_notes;
+	      break;
+	    }
+	  else
+	    {
+	      gregorio_free_one_note (&current_note);	// automatically sets first_note to the next note
+	    }
+	}
+      gregorio_go_to_first_note (&current_note);
       // finally we set the just added glyph first_note to current_note
       (*last_glyph)->first_note = current_note;
-   }
+    }
 }
 
 // a small function to automatically determine the pitch of a custo : it is the pitch of the next note, but we must take care of the clef changes, as custo are (normally and for now) only present before clef changes.
+// TODO: there may be a side effect with the flated keys...
 
 char
-libgregorio_gabc_determine_custo_pitch (gregorio_note *current_note, int current_key)
+gabc_determine_custo_pitch (gregorio_note * current_note,
+					int current_key)
 {
   int pitch_difference = 0;
   int newkey;
-  while(current_note)
+  while (current_note)
     {
-      if (current_note->type == GRE_C_KEY_CHANGE)
-        {
-          newkey = gregorio_calculate_new_key (C_KEY, current_note->pitch - 48);
-          pitch_difference = newkey - current_key;
-        }
-      if (current_note->type == GRE_F_KEY_CHANGE)
-        {
-          newkey = gregorio_calculate_new_key (C_KEY, current_note->pitch - 48);
-          pitch_difference = newkey - current_key;
-        }
+      if (current_note->type == GRE_C_KEY_CHANGE
+	  || current_note->type == GRE_C_KEY_CHANGE_FLATED)
+	{
+	  newkey =
+	    gregorio_calculate_new_key (C_KEY, current_note->pitch - 48);
+	  pitch_difference = newkey - current_key;
+	}
+      if (current_note->type == GRE_F_KEY_CHANGE
+	  || current_note->type == GRE_F_KEY_CHANGE_FLATED)
+	{
+	  newkey =
+	    gregorio_calculate_new_key (F_KEY, current_note->pitch - 48);
+	  pitch_difference = newkey - current_key;
+	}
       if (current_note->type == GRE_NOTE)
-        {
-          return current_note -> pitch - (char) pitch_difference;
-        }
-      current_note = current_note -> next_note;
+	{
+	  return current_note->pitch - (char) pitch_difference;
+	}
+      current_note = current_note->next;
     }
   return 'g';
 }
@@ -181,7 +205,7 @@ libgregorio_gabc_determine_custo_pitch (gregorio_note *current_note, int current
  * 
  * Function called with a list of gregorio_notes as argument, this
  * list is determined from gabc notation by the function
- * libgregorio_gabc_det_notes_from_string.
+ * gabc_det_notes_from_string.
  * 
  * In this function we create a list of glyphs, by determining their
  * type according to the sequence of notes (we look at their height,
@@ -216,7 +240,8 @@ libgregorio_gabc_determine_custo_pitch (gregorio_note *current_note, int current
 // this function updates current_key with the new values (with clef changes)
 
 gregorio_glyph *
-libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *current_key)
+gabc_det_glyphs_from_notes (gregorio_note * current_note,
+					int *current_key)
 {
   // the first note of the current glyph, to be able to close it well:
   // later we will cut the link (next_notes and previous_note) between
@@ -231,6 +256,7 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
   char current_glyph_type = G_UNDETERMINED;
   char next_glyph_type = G_UNDETERMINED;
   char last_pitch = USELESS_VALUE;
+  char additional_infos = 0;	// a variable for the signs of bars and to tell if a key is flatted or not
   gregorio_note *next_note = NULL;
 
   // determination of end of glyphs, see comments on
@@ -249,36 +275,113 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 
   while (current_note)
     {
-      next_note = current_note->next_note;
+      next_note = current_note->next;
       if (current_note->type != GRE_NOTE)
 	{
 	  if (current_glyph_type != G_UNDETERMINED)
 	    {
 	      close_glyph (&last_glyph, next_glyph_type,
 			   &current_glyph_first_note, liquescentia,
-			   current_note->previous_note);
+			   current_note->previous);
 	      current_glyph_type = G_UNDETERMINED;
 	      liquescentia = L_NO_LIQUESCENTIA;
 	    }
 	  if (current_note->type == GRE_C_KEY_CHANGE)
-        {
-          *current_key = gregorio_calculate_new_key (C_KEY, current_note->pitch - 48);
-        }
-      if (current_note->type == GRE_F_KEY_CHANGE)
-        {
-          *current_key = gregorio_calculate_new_key (C_KEY, current_note->pitch - 48);
-        }
+	    {
+	      *current_key =
+		gregorio_calculate_new_key (C_KEY, current_note->pitch - 48);
+	    }
+	  if (current_note->type == GRE_F_KEY_CHANGE)
+	    {
+	      *current_key =
+		gregorio_calculate_new_key (F_KEY, current_note->pitch - 48);
+	    }
+	  if (current_note->type == GRE_C_KEY_CHANGE_FLATED)
+	    {
+	      *current_key =
+		gregorio_calculate_new_key (C_KEY, current_note->pitch - 48);
+	      gregorio_add_special_as_glyph (&last_glyph, GRE_C_KEY_CHANGE,
+					     current_note->pitch, FLAT_KEY, NULL);
+	      current_glyph_first_note = current_note->next;
+	      gregorio_free_one_note (&current_note);
+	      last_pitch = USELESS_VALUE;
+	      continue;
+	    }
+	  if (current_note->type == GRE_F_KEY_CHANGE_FLATED)
+	    {
+	      *current_key =
+		gregorio_calculate_new_key (F_KEY, current_note->pitch - 48);
+	      gregorio_add_special_as_glyph (&last_glyph, GRE_F_KEY_CHANGE,
+					     current_note->pitch, FLAT_KEY, NULL);
+	      current_glyph_first_note = current_note->next;
+	      gregorio_free_one_note (&current_note);
+	      last_pitch = USELESS_VALUE;
+	      continue;
+	    }
 	  if (current_note->type == GRE_CUSTO)
 	    {
 	      current_note->pitch =
-	          libgregorio_gabc_determine_custo_pitch (current_note->next_note, *current_key);
+		gabc_determine_custo_pitch (current_note->
+							next,
+							*current_key);
+	    }
+	  // we calculate the signs of the bars
+	  if (current_note->type == GRE_BAR)
+	    {
+	      if (current_note->signs == _V_EPISEMUS)
+		{
+		  switch (current_note->rare_sign)
+		    {
+		    case _ICTUS_A:
+		      additional_infos = _V_EPISEMUS_ICTUS_A;
+		      break;
+		    case _ICTUS_T:
+		      additional_infos = _V_EPISEMUS_ICTUS_T;
+		      break;
+		    default:
+		      additional_infos = _V_EPISEMUS;
+		      break;
+		    }
+		}
+	      else
+		{
+		  additional_infos = current_note->rare_sign;
+		}
+		if (current_note->h_episemus_type != H_NO_EPISEMUS)
+      {
+        switch (additional_infos)
+          {
+		    case _ICTUS_A:
+		      additional_infos = _H_EPISEMUS_ICTUS_A;
+		      break;
+		    case _ICTUS_T:
+		      additional_infos = _H_EPISEMUS_ICTUS_T;
+		      break;
+		    case _V_EPISEMUS:
+		      additional_infos = _V_EPISEMUS_H_EPISEMUS;
+		      break;
+		    case _V_EPISEMUS_ICTUS_T:
+		      additional_infos = _V_EPISEMUS_H_EPISEMUS_ICTUS_T;
+		      break;
+		    case _V_EPISEMUS_ICTUS_A:
+		      additional_infos = _V_EPISEMUS_H_EPISEMUS_ICTUS_T;
+		      break;
+		    default:
+		      additional_infos = _H_EPISEMUS;
+		      break;
+          }
+      }
 	    }
 	  gregorio_add_special_as_glyph (&last_glyph, current_note->type,
-					    current_note->pitch);
-	  current_glyph_first_note = current_note->next_note;
+					 current_note->pitch,
+					 additional_infos, current_note->texverb);
+	  current_glyph_first_note = current_note->next;
+	  current_note->texverb = NULL;
 	  gregorio_free_one_note (&current_note);
-      last_pitch = USELESS_VALUE;
+	  last_pitch = USELESS_VALUE;
+	  additional_infos = 0;
 	  //TODO : change behaviour here for flat and natural
+	  //UPDATE: what does this TODO mean?...
 	  continue;
 	}
       /* first we do what must be done with liquescentia */
@@ -290,7 +393,7 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 	      /* if it is not the first glyph */
 	      close_glyph (&last_glyph, current_glyph_type,
 			   &current_glyph_first_note,
-			   liquescentia, current_note->previous_note);
+			   liquescentia, current_note->previous);
 	      current_glyph_type = G_UNDETERMINED;
 	    }
 	  liquescentia = L_INITIO_DEBILIS;
@@ -298,8 +401,8 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 
       next_glyph_type =
 	gregorio_add_note_to_a_glyph (current_glyph_type,
-					 current_note->pitch, last_pitch,
-					 current_note->shape, &end_of_glyph);
+				      current_note->pitch, last_pitch,
+				      current_note->shape, &end_of_glyph);
 
 
 // patch to have good shapes in the special cases of pes quadratum and
@@ -338,11 +441,10 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 		      current_note->shape = S_PUNCTUM_INCLINATUM_AUCTUS;
 		    }
 
-		  if (current_note->next_note
-		      && current_note->next_note->shape ==
+		  if (current_note->next
+		      && current_note->next->shape ==
 		      S_PUNCTUM_INCLINATUM
-		      && current_note->next_note->liquescentia ==
-		      L_DEMINUTUS)
+		      && current_note->next->liquescentia == L_DEMINUTUS)
 		    {
 		      last_pitch = current_note->pitch;
 		      current_note = next_note;
@@ -361,7 +463,7 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 	case DET_END_OF_PREVIOUS:
 	  close_glyph (&last_glyph, current_glyph_type,
 		       &current_glyph_first_note, liquescentia,
-		       current_note->previous_note);
+		       current_note->previous);
 	  current_glyph_type = next_glyph_type;
 	  liquescentia = L_NO_LIQUESCENTIA;
 	  last_pitch = USELESS_VALUE;
@@ -370,7 +472,7 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 	    //not an initio debilis, because we considered it in the first
 	    //part...
 	    {
-	    	      //special cases of the punctum inclinatum deminutus and auctus
+	      //special cases of the punctum inclinatum deminutus and auctus
 	      if (current_note->shape == S_PUNCTUM_INCLINATUM)
 		{
 		  if (current_note->liquescentia == L_DEMINUTUS)
@@ -383,11 +485,10 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 		    {
 		      current_note->shape = S_PUNCTUM_INCLINATUM_AUCTUS;
 		    }
-		  if (current_note->next_note
-		      && current_note->next_note->shape ==
+		  if (current_note->next
+		      && current_note->next->shape ==
 		      S_PUNCTUM_INCLINATUM
-		      && current_note->next_note->liquescentia ==
-		      L_DEMINUTUS)
+		      && current_note->next->liquescentia == L_DEMINUTUS)
 		    {
 		      last_pitch = current_note->pitch;
 		      current_note = next_note;
@@ -411,7 +512,7 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 	default:		//case DET_END_OF_BOTH:
 	  close_glyph (&last_glyph, current_glyph_type,
 		       &current_glyph_first_note, liquescentia,
-		       current_note->previous_note);
+		       current_note->previous);
 	  current_glyph_type = G_UNDETERMINED;
 	  liquescentia = L_NO_LIQUESCENTIA;
 	  close_glyph (&last_glyph, next_glyph_type,
@@ -445,7 +546,7 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
  * glyph, or will it be the first note of another glyph ?
  * 
  * current_glyph_type is the type of the current_glyph (current_glyph
- * as meant in libgregorio_gabc_det_glyphs_from_notes).  curent_pitch
+ * as meant in gabc_det_glyphs_from_notes).  curent_pitch
  * is the height of the note that we want to "add" to the glyph.
  * last_pitch is the height of the last note of current_glyph.  shape
  * is the shape of the note we want to "add" to the glyph.
@@ -481,8 +582,7 @@ libgregorio_gabc_det_glyphs_from_notes (gregorio_note * current_note, int *curre
 
 char
 gregorio_add_note_to_a_glyph (char current_glyph_type, char current_pitch,
-				 char last_pitch, char shape,
-				 char *end_of_glyph)
+			      char last_pitch, char shape, char *end_of_glyph)
 {
 
 // next glyph type is the type of the glyph that will be returned (the
@@ -508,9 +608,10 @@ gregorio_add_note_to_a_glyph (char current_glyph_type, char current_pitch,
     case S_PUNCTUM_CAVUM:
     case S_LINEA_PUNCTUM:
     case S_LINEA_PUNCTUM_CAVUM:
+    case S_LINEA:
       next_glyph_type = G_PUNCTUM;
-	  *end_of_glyph = DET_END_OF_BOTH;
-	  break;
+      *end_of_glyph = DET_END_OF_BOTH;
+      break;
     case S_PUNCTUM:
 /* we determine here the shape of the thing if it is made of puncta */
       if (current_pitch == last_pitch)
@@ -625,6 +726,10 @@ gregorio_add_note_to_a_glyph (char current_glyph_type, char current_pitch,
       *end_of_glyph = DET_END_OF_PREVIOUS;
       next_glyph_type = G_PUNCTUM;
       break;
+    case S_ORISCUS_DEMINUTUS:
+      *end_of_glyph = DET_END_OF_PREVIOUS;
+      next_glyph_type = G_PUNCTUM;
+      break;
     case S_QUILISMA:
       *end_of_glyph = DET_END_OF_PREVIOUS;
       next_glyph_type = G_PUNCTUM;
